@@ -29,6 +29,7 @@
 
 // Apt includes
 #include <apt-pkg/algorithms.h>
+#include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/depcache.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/fileutl.h>
@@ -93,12 +94,15 @@ public:
     // Xapian
     time_t xapianTimeStamp;
     Xapian::Database xapianDatabase;
+    bool xapianIndexExists;
 
     // DBus
     QDBusServiceWatcher *watcher;
     OrgKubuntuQaptworkerInterface *worker;
 
     Config *config;
+    bool isMultiArch;
+    QString nativeArch;
 
     bool writeSelectionFile(const QString &file, const QString &path) const;
     void setWorkerLocale();
@@ -165,6 +169,8 @@ bool Backend::init()
 
     d->cache = new Cache(this);
     d->config = new Config(this);
+    d->nativeArch = config()->readEntry(QLatin1String("APT::Architecture"),
+                                        QLatin1String(""));
     reloadCache();
     openXapianIndex();
 
@@ -200,6 +206,10 @@ void Backend::reloadCache()
     int count = 0;
     QSet<Group> groupSet;
 
+    d->isMultiArch = APT::Configuration::getArchitectures(false).size() > 1;
+    bool pkgMultiArch;
+    QString arch;
+
     pkgCache::PkgIterator iter;
     for (iter = depCache->PkgBegin(); !iter.end(); ++iter) {
         if (!iter->VersionList) {
@@ -207,6 +217,18 @@ void Backend::reloadCache()
         }
 
         Package *pkg = new Package(this, depCache, d->records, iter);
+
+        if (d->isMultiArch) {
+            // Do not add non-multiarch packages
+            pkgMultiArch = pkg->isMultiArchEnabled();
+            arch = pkg->architecture();
+
+            if (!pkgMultiArch && (arch != d->nativeArch) && (arch != QLatin1String("all"))) {
+                delete pkg;
+                continue;
+            }
+        }
+
         d->packagesIndex[iter->ID] = count;
         d->packages.append(pkg);
         ++count;
@@ -586,6 +608,13 @@ GroupList Backend::availableGroups() const
     return groupList;
 }
 
+bool Backend::isMultiArchEnabled() const
+{
+    Q_D(const Backend);
+
+    return d->isMultiArch;
+}
+
 bool Backend::areChangesMarked() const
 {
     return (toInstallCount() + toRemoveCount());
@@ -614,12 +643,14 @@ bool Backend::xapianIndexNeedsUpdate() const
    // If the cache has been modified after the xapian timestamp, we need to rebuild
    QDateTime aptCacheTime = QFileInfo(QLatin1String(_config->FindFile("Dir::Cache::pkgcache").c_str())).lastModified();
    QDateTime dpkgStatusTime = QFileInfo(QLatin1String("/var/lib/dpkg/status")).lastModified();
+
+   bool outdated = false;
    if (d->xapianTimeStamp < aptCacheTime.toTime_t() ||
        d->xapianTimeStamp < dpkgStatusTime.toTime_t()) {
-      return true;
+       outdated = true;
    }
 
-   return false;
+   return (outdated || (!d->xapianIndexExists));
 }
 
 bool Backend::openXapianIndex()
@@ -632,9 +663,12 @@ bool Backend::openXapianIndex()
     try {
         d->xapianDatabase.add_database(Xapian::Database("/var/lib/apt-xapian-index/index"));
     } catch (Xapian::DatabaseOpeningError) {
+        d->xapianIndexExists = false;
         return false;
     };
 
+    d->xapianIndexExists = true;
+
     return true;
 }
 
@@ -814,6 +848,7 @@ void Backend::commitChanges()
 
     Q_FOREACH (const Package *package, d->packages) {
         int flags = package->state();
+        std::string fullName = package->packageIterator()->FullName();
         // Cannot have any of these flags simultaneously
         int status = flags & (Package::ToKeep |
                               Package::NewInstall |
@@ -824,26 +859,26 @@ void Backend::commitChanges()
         switch (status) {
            case Package::ToKeep:
                if (flags & Package::Held) {
-                   packageList.insert(package->name(), Package::Held);
+                   packageList.insert(fullName.c_str(), Package::Held);
                }
                break;
            case Package::NewInstall:
-               packageList.insert(package->name(), Package::ToInstall);
+               packageList.insert(fullName.c_str(), Package::ToInstall);
                break;
            case Package::ToReInstall:
-               packageList.insert(package->name(), Package::ToReInstall);
+               packageList.insert(fullName.c_str(), Package::ToReInstall);
                break;
            case Package::ToUpgrade:
-               packageList.insert(package->name(), Package::ToUpgrade);
+               packageList.insert(fullName.c_str(), Package::ToUpgrade);
                break;
            case Package::ToDowngrade:
-               packageList.insert(package->name() % ',' % package->availableVersion(), Package::ToDowngrade);
+               packageList.insert(QString(fullName.c_str()) % ',' % package->availableVersion(), Package::ToDowngrade);
                break;
            case Package::ToRemove:
                if(flags & Package::ToPurge) {
-                   packageList.insert(package->name(), Package::ToPurge);
+                   packageList.insert(fullName.c_str(), Package::ToPurge);
                } else {
-                   packageList.insert(package->name(), Package::ToRemove);
+                   packageList.insert(fullName.c_str(), Package::ToRemove);
                }
                break;
         }
