@@ -76,7 +76,10 @@ QAptWorker::QAptWorker(int &argc, char **argv)
         return;
     }
 
-    QTimer::singleShot(30000, this, SLOT(quit()));
+    m_timeout = new QTimer(this);
+    connect(m_timeout, SIGNAL(timeout()), this, SLOT(quit()));
+    m_timeout->setSingleShot(true);
+    m_timeout->start(30000);
 }
 
 QAptWorker::~QAptWorker()
@@ -271,7 +274,17 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
                 m_cache->depCache()->MarkKeep(iter, false);
                 m_cache->depCache()->SetReInstall(iter, false);
                 break;
-            case QApt::Package::ToUpgrade:
+            case QApt::Package::ToUpgrade: {
+                bool fromUser = !(State.Flags & pkgCache::Flag::Auto);
+                m_cache->depCache()->MarkInstall(iter, true, 0, fromUser);
+                if (!State.Install() || m_cache->depCache()->BrokenCount() > 0) {
+                    pkgProblemResolver Fix(m_cache->depCache());
+                    Fix.Clear(iter);
+                    Fix.Protect(iter);
+                    Fix.Resolve(true);
+                }
+                break;
+            }
             case QApt::Package::ToInstall:
                 m_cache->depCache()->MarkInstall(iter, true);
                 if (!State.Install() || m_cache->depCache()->BrokenCount() > 0) {
@@ -532,6 +545,7 @@ void QAptWorker::installDebFile(const QString &fileName)
         emit workerFinished(false);
         return;
     }
+    m_timeout->stop();
 
     emit workerStarted();
 
@@ -587,6 +601,8 @@ void QAptWorker::dpkgFinished(int exitCode, QProcess::ExitStatus exitStatus)
         emit workerEvent(QApt::DebInstallFinished);
     }
 
+    m_timeout->start();
+
     emit workerFinished(!exitStatus);
     delete m_dpkgProcess;
     m_dpkgProcess = 0;
@@ -607,6 +623,7 @@ void QAptWorker::setAnswer(const QVariantMap &answer)
 
 void QAptWorker::updateXapianIndex()
 {
+    m_timeout->stop();
     emit workerEvent(QApt::XapianUpdateStarted);
 
 
@@ -635,6 +652,7 @@ void QAptWorker::xapianUpdateFinished(bool result)
                                             QLatin1String("UpdateFinished"), this, SLOT(xapianUpdateFinished(bool)));
 
     emit workerEvent(QApt::XapianUpdateFinished);
+    m_timeout->start();
 }
 
 bool QAptWorker::writeFileToDisk(const QString &contents, const QString &path)
